// =============================================================================
// Filter.cpp: Main program of console application Filter
// Filter application, (c) lars.staurset@statkart.no 2003
// Purpose of application: Some processing of text files
// =============================================================================

#include <iostream>
#include <string>
#include "Typedefs.h"
#include "Copy.h"
#include "Mbox.h"
#include "Exception.h"

// List of commands
enum CommandType {eNone, eCopy, eAttachment};

// Display a help text
void Help (const string sProgramName)
{
    cout << "Usage:\n";
    cout << "  " << sProgramName << " [/a|/c|/h] [/q|/v] infile [outfile]\n";
    // I know they are streams, but "file" is more understandable to the average user
    cout << "Commands:\n";
    cout << "  /a : remove attachments from mbox folder specified by infile\n";
    cout << "  /c : copy infile to outfile (default command)\n";
    cout << "  /h : display this help text\n";
    cout << "If outfile is missing, a default name will be supplied.\n";
    cout << "Options:\n";
    cout << "  /q : quiet mode, i.e. no detailed console output\n";
    cout << "  /v : verbose mode, i.e. show complete headers of every message\n";
    cout << "Console output can be redirected to a file if your OS permits.\n";
    cout << "YOU ARE USING THIS PROGRAM AND ITS OUTPUT AT YOUR OWN RISK,\n";
    cout << "in other words: If you're disappointed, tell me but don't sue me.\n";
}

// Main entry point
int main (int argc, char *argv[])
{
    // Presentation
    cout << "Filter v. 1.02 -- (c) lars.staurset@statkart.no 2003\n";

    // Local variables
    string sInStream, sOutStream;
    const string sError1 = " conflicts with another command or option";
    const string sError2 = "Unknown command or option: ";
    CommandType eCommand = eNone;
    OutputLevel eOutputLevel = OL_Normal;
    bool bHelp = false;
    bool bSuccess = true;

    // Parse command line
    // Commands or options to be prefixed with / or -
    // Other arguments will be interpreted as input stream and optionally output stream
    try {
        for (int iArg = 1; iArg < argc; ++iArg) {
            if (*argv[iArg] == '/' || *argv[iArg] == '-') {

                // Command or option
                switch (*(argv[iArg]+1)) {

                // Commands

                case 'a':
                case 'A':
                    if (eCommand == eNone || eCommand == eAttachment)
                        eCommand = eAttachment;
                    else
                        throw cException (argv[iArg] + sError1);
                    break;
                case 'c':
                case 'C':
                    if (eCommand == eNone || eCommand == eCopy)
                        eCommand = eCopy;
                    else
                        throw cException (argv[iArg] + sError1);
                    break;

                // Help

                case 'h':
                case 'H':
                    bHelp = true;
                    break;

                // Output level

                case 'd':
                    if (eOutputLevel < OL_Debug)
                        eOutputLevel = OL_Debug;
                    break;
                case 'D':
                    if (eOutputLevel < OL_Dump)    // Overrides all other output options
                        eOutputLevel = OL_Dump;
                    break;
                case 'q':
                case 'Q':
                    if (eOutputLevel == OL_Normal)
                        eOutputLevel = OL_Quiet;
                    else if (eOutputLevel == OL_Verbose)
                        throw cException (argv[iArg] + sError1);
                    break;
                case 'v':
                case 'V':
                    if (eOutputLevel == OL_Normal)
                        eOutputLevel = OL_Verbose;
                    else if (eOutputLevel == OL_Quiet)
                        throw cException (argv[iArg] + sError1);
                    break;

                default:
                    throw cException (sError2 + argv[iArg]);
                }

            } else {

                // File name
                if (sInStream.empty())
                    sInStream = argv[iArg];
                else {
                    if (sOutStream.empty())
                        sOutStream = argv[iArg];
                    else
                        throw cException ("Too many file args -- only infile and outfile are supported");
                }
            }
        }

        // Help user
        if (bHelp) {
            Help (argv[0]);
            return 0;
        }

        // Check that commands etc. make sense
        if (eCommand == eNone)
            eCommand = eCopy;       // Default command
        if (sInStream.empty())
            throw cException ("Filename missing");

    } catch (const cException cU) {
        // Early error exit
        cout << cU.GetText() << "\n";
        Help (argv[0]);
	     return 1;
    }

    // Handle command
    cCommand *pCommand = NULL;
    try {
        // Preprocessing: Set outstream name if missing; create command object
        bool bOutstreamSpecified = !sOutStream.empty();
        switch (eCommand) {
            case eCopy:
                if (!bOutstreamSpecified)
                    sOutStream = "Copy of " + sInStream;
                pCommand = new cCopy (sInStream, sOutStream, bOutstreamSpecified, eOutputLevel);
                break;
            case eAttachment:
                if (!bOutstreamSpecified)
                    sOutStream = sInStream + "_Small";
                pCommand = new cMbox (sInStream, sOutStream, bOutstreamSpecified, eOutputLevel);
                break;
        }

    } catch (const cException cE) {
        // Error exit on creation
        cout << cE.GetText() << "\n";
        bSuccess = false;
    }

    // Execute command
    if (bSuccess)
        bSuccess = pCommand->Execute();

    // Normal and error exit
    if (pCommand)
        delete pCommand;
    return bSuccess ? 0 : 1;
}
