// =============================================================================
// Header.h: Declarations of class cHeader
// Filter application, (c) lars.staurset@statkart.no 2003
// Purpose: Base class for specialized header classes
// =============================================================================

#pragma once
#include <list>
#include <string>
#include "OutStream.h"

typedef struct {
    list<string>::iterator iPos;
    size_t nPos;
    size_t nLength;
} HeaderPosition;

class cHeader
{
    // =========================================================================
    // Public functions
    // =========================================================================
public:
    // ---------------------------------------------------------------------------
    // Class administration (construction, destruction, etc.)
    // ---------------------------------------------------------------------------
    cHeader (
        const OutputLevel);     // Output level, see Typedefs.h
    virtual ~cHeader(void);

    // ---------------------------------------------------------------------------
    // Data processing
    // ---------------------------------------------------------------------------
    // Process a header line
    // Success: Line is stored in buffer
    // In some cases line is appended to previous line
    // Failure: N/A
    bool ProcessLine (      // returns true if more header lines are expected
        const string);          // Line to be processed

    // Save header to stream
    // Success: The stored header lines are written to outstream
    // Failure: Throws a cException on file errors
    void Save (
        cOutStream *);          // Output stream

    // =========================================================================
    // Protected functions
    // =========================================================================

    // Does a string start with a specific substring?
    // Case-sensitive
    // Success: Substring either found or not
    // Failure: N/A
    bool StringStartsWith ( // true if yes
        const string,           // String
        const string,           // Substring (key)
        size_t &)               // Starting point in string for further search,
                                // unchanged unless function returns true
        const;

    // Find a key/value pair in a line
    // Format: some text and then key=value or key="value"
    // Success: Found key, returns value
    // Failure: No key, returns empty string
    // Total failure: Didn't find delimiter (= or "), throws a cException
    size_t Find (           // returns length of value
        const string,           // Line, preferably lower-case
        const string,           // Key, preferably lower-case
        const size_t,           // Starting point in line for search
        size_t &)               // Returns starting point of value if found
        const;

    // Find some value in a line
    // Format: some text and then value delimited by space, tab, or end-of-line
    // Success: Delimiter found, clients gets starting point and length of value
    // Failure: No delimiter, throws a cException
    size_t Find (           // returns length of value
        const string,           // Line, preferably lower-case
        const size_t,           // Starting point in line for search
        size_t &)               // Returns starting point of value if found
        const;

    // =========================================================================
    // Private functions
    // =========================================================================
private:

    // =========================================================================
    // Protected data members
    // =========================================================================
protected:
    // ---------------------------------------------------------------------------
    // Members passed from client class
    // ---------------------------------------------------------------------------
    OutputLevel m_eOutputLevel;    // Output level, see Typedefs.h

    // ---------------------------------------------------------------------------
    // Members proper to this class
    // ---------------------------------------------------------------------------
    list<string> m_lsLines;     // Header contents

    // =========================================================================
    // Private data members
    // =========================================================================
private:
    // ---------------------------------------------------------------------------
    // Members passed from client class
    // ---------------------------------------------------------------------------

    // ---------------------------------------------------------------------------
    // Members proper to this class
    // ---------------------------------------------------------------------------
    bool m_bMayAppendLine;      // True if previous line permits current line to append
};

