// =============================================================================
// MessageHeader.cpp: Implementation of class cMessageHeader
// Filter application, (c) lars.staurset@statkart.no 2003
// =============================================================================

#include <iostream>
#include "MessageHeader.h"
#include "Exception.h"

// =========================================================================
// Public functions
// =========================================================================

// ---------------------------------------------------------------------------
// Class administration (construction, destruction, etc.)
// ---------------------------------------------------------------------------
// Constructor
cMessageHeader::cMessageHeader (const OutputLevel ceOutputLevel)
: cHeader (ceOutputLevel)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessageHeader::cMessageHeader\n";
    m_bDeleted = false;
    m_eContentType = CT_Unknown;
}

cMessageHeader::~cMessageHeader(void)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessageHeader::~cMessageHeader\n";
}

// ---------------------------------------------------------------------------
// Data processing
// ---------------------------------------------------------------------------

// Process a header line
bool cMessageHeader::ProcessLine (const string csLine)
{
    // Deleted message?
    // This information can be requested by client before complete header is analyzed
    if (!m_bDeleted)
        SetDeleted (csLine);

    return cHeader::ProcessLine (csLine);
}

// Analyze a header
void cMessageHeader::Analyze ()
{
    // Iterator loop
    list<string>::const_iterator ciPos;
    for (ciPos = m_lsLines.begin(); ciPos != m_lsLines.end(); ++ciPos) {

        if (m_eOutputLevel >= OL_Debug)
            cout << "cMessageHeader::Analyze:\'" << *ciPos << "\'\n";

        // Convert line to lower case for comparisons
        string sLowerCaseLine = *ciPos;
        for (unsigned int un = 0; un < sLowerCaseLine.length(); ++un) {
            char ch = sLowerCaseLine[un];
            sLowerCaseLine[un] = tolower (ch);
        }

        // Line to be displayed?
        Display (sLowerCaseLine, *ciPos);

        // Find Content-Type
        size_t nPos = 0;
        size_t nPosIgnored = 0;
        if (StringStartsWith (sLowerCaseLine, "content-type: multipart/related", nPos)) {
            m_eContentType = CT_Related;

        } else if (StringStartsWith (sLowerCaseLine, "content-type: multipart/mixed", nPos)) {
            m_eContentType = CT_Mixed;

        } else if (StringStartsWith (sLowerCaseLine, "content-type: multipart/report", nPos)) {
            m_eContentType = CT_Report;

        // I ignore multipart/alternative, assuming that type normally won't
        // contain attachments. Some virus messages break that rule, and it
        // would be nice to remove those attachments, but such messages appear
        // to be malformed in several ways, and I haven't got the time to
        // analyze and handle every possibility.
        } else if (StringStartsWith (sLowerCaseLine, "content-type: multipart/alternative", nPosIgnored)) {
            m_eContentType = CT_Alternative;

        // Should other content-types be handled?
        } else if (StringStartsWith (sLowerCaseLine, "content-type: multipart/", nPosIgnored)) {
            if (m_eOutputLevel >= OL_Normal)
                cout << "Ignoring \'" << *ciPos << "\'\n";
        }

        // Find boundary, if any
        if (nPos) {
            size_t nLength = Find (sLowerCaseLine, "boundary", nPos, nPos);
            if (nLength) {
                m_sBoundary = ciPos->substr (nPos, nLength);
                if (m_eOutputLevel >= OL_Debug)
                    cout << "cMessageHeader::Analyze: m_sBoundary=\'" << m_sBoundary << "\'\n";
            } else {
                throw cException ("Syntax error or unimplemented format - cannot find \'boundary\' in line \'" + *ciPos + "\'");
            }
        }
    }
}

// Display some info from a header line
void cMessageHeader::Display (const string csLowerCaseLine, const string csRightCaseLine) const
{
    bool bDisplay = false;
    if (m_eOutputLevel >= OL_Verbose)
        bDisplay = true;
    else if (m_eOutputLevel >= OL_Normal) {
        if (csLowerCaseLine.substr(0,6) == "date: " ||
            csLowerCaseLine.substr(0,6) == "from: " ||
            csLowerCaseLine.substr(0,9) == "subject: ")
            bDisplay = true;
    }
    if (bDisplay)
        cout << csRightCaseLine << endl;
}

// =========================================================================
// Private functions
// =========================================================================

// Determine whether current line indicates deletes message
// X-Mozilla-Status of message:
// bit  0: 1 for read
// bit  1: 1 for replied
// bit  2: 1 for flag
// bit  3: 1 for deleted (the only one used here)
// bit 25: 1 for forwarded
void cMessageHeader::SetDeleted (const string csLine)
{
    const unsigned int MASK_DELETED = 8;    // bit 3 set
    unsigned long XMozillaStatus;
    if (GetHex (csLine, "X-Mozilla-Status: ", &XMozillaStatus)) {
        if (XMozillaStatus & MASK_DELETED)
            m_bDeleted = true;
        if (m_eOutputLevel >= OL_Debug)
            cout << "cMessageHeader::SetDeleted: m_bDeleted = " << m_bDeleted << endl;
    }
}

// Look for hex value in current line
bool cMessageHeader::GetHex (const string csLine, const string csKey, unsigned long *pulValue) const
{
    size_t nKeyLength = 0;
    bool bFound = StringStartsWith (csLine, csKey, nKeyLength);
    if (bFound) {
        if (m_eOutputLevel >= OL_Debug)
            cout << "cMessageHeader::GetHex-i: " << csLine << endl;
        string sValue = csLine.substr (nKeyLength);
        *pulValue = strtoul (sValue.c_str(), NULL, 16);
        if (m_eOutputLevel >= OL_Debug)
            cout << "cMessageHeader::GetHex-o: " << hex << sValue << " " << *pulValue << dec << endl;
    }
    return bFound;
}

