// =============================================================================
// Part.cpp: Implementation of class cPart
// Filter application, (c) lars.staurset@statkart.no 2003
// =============================================================================

#include <iostream>
#include "Part.h"
#include "Exception.h"

// =========================================================================
// Public functions
// =========================================================================

// ---------------------------------------------------------------------------
// Class administration (construction, destruction, etc.)
// ---------------------------------------------------------------------------
// Constructor
cPart::cPart (const ContentType ceContentType, const OutputLevel ceOutputLevel,
              cOutStream *pOutStream)
{
    // Store parameters
    m_eContentType = ceContentType;
    m_eOutputLevel = ceOutputLevel;
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPart::cPart\n";
    m_pOutStream = pOutStream;

    // Create header object since part always starts with a header
    m_pPartHeader = new cPartHeader (m_eContentType == CT_Report, m_eOutputLevel);
    m_bInBodyToBeRemoved = false;
    m_bRemoved = false;
}

cPart::~cPart(void)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPart::~cPart\n";
}

// ---------------------------------------------------------------------------
// Data processing
// ---------------------------------------------------------------------------

// Process a line that belongs to a part
bool cPart::ProcessLine (const string csLine)
{
    bool bRemovedEarlier = false;
    if (m_pPartHeader) {

        // In header: Add line to temp header storage
        if (!m_pPartHeader->ProcessLine (csLine)) {

            // Header ends here - analyze it
            bRemovedEarlier = AnalyzeHeader();
        }

    // In body
    } else if (m_bInBodyToBeRemoved) {
        // In body to be removed
        if (m_eOutputLevel == OL_Dump)
            cout << "cPart::ProcessLine:Still in body of part to be removed\n";
    } else {
        // In body to be kept
        if (m_eOutputLevel == OL_Dump)
            cout << "cPart::ProcessLine:In body of part to be kept\n";
        m_pOutStream->WriteLine (csLine);
    }
    return bRemovedEarlier;
}

// =========================================================================
// Private functions
// =========================================================================

bool cPart::AnalyzeHeader ()
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPart::AnalyzeHeader-i\n";
    bool bRemovedEarlier = false;

    if (m_pPartHeader->Analyze ()) {

        // Unremovable part, save header
        if (m_eOutputLevel >= OL_Normal)
            cout << "Unremovable part: " << m_pPartHeader->GetContentType() << "\n";
        m_pPartHeader->Save (m_pOutStream);

    } else if (m_eContentType == CT_Report) {

        // In report part to be removed
        bRemovedEarlier = RemoveReportPart();

    } else {

        // In attachment part to be removed
        bRemovedEarlier = RemoveAttachment();
    }

    // Header finished
    if (m_eOutputLevel >= OL_Debug && m_pPartHeader != NULL)
        cout << "cPart::ProcessLine:Leaving part header\n";
    delete m_pPartHeader;
    m_pPartHeader = NULL;

    if (m_eOutputLevel >= OL_Debug)
        cout << "cPart::AnalyzeHeader-o: bRemovedEarlier=" << bRemovedEarlier << "\n";
    return bRemovedEarlier;
}

// Do what's needed to remove this part of a report-type message
bool cPart::RemoveReportPart ()
{
    // Removable part of a report
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPart::RemoveReportPart-i: \'" << m_pPartHeader->GetContentType() << "\'\n";

    // Prepare and save header info
    bool bRemovedEarlier = m_pPartHeader->PrepareReportPartHeader();
    m_pPartHeader->Save (m_pOutStream);
    if (bRemovedEarlier) {
        // Part already removed, probably by this program
        if (m_eOutputLevel >= OL_Normal)
            cout << "Part was already removed: " << m_pPartHeader->GetFilename() << "\n";
    } else {
        // Found part to be removed now
        m_bRemoved = true;
        if (m_eOutputLevel >= OL_Debug && !m_bInBodyToBeRemoved)
            cout << "cPart::ProcessLine:Entering nonattachment part body to be removed\n";
        m_bInBodyToBeRemoved = true;

        // Text to replace part body
        m_pOutStream->WriteLine (m_pPartHeader->GetContentType());
        if (m_eOutputLevel >= OL_Normal)
            cout << m_pPartHeader->GetContentType() << "\n";
    }

    if (m_eOutputLevel >= OL_Debug)
        cout << "cPart::RemoveReportPart-o:bRemovedEarlier=" << bRemovedEarlier << "\n";
    return bRemovedEarlier;
}

// Do what's needed to remove this attachment of a nonreport message
bool cPart::RemoveAttachment ()
{
    // Removable part with a file attachment
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPart::RemoveAttachment-i: \'" << m_pPartHeader->GetFilename() << "\'\n";
    bool bRemovedEarlier = false;

    // First prepare header info
    if (m_pPartHeader->PrepareAttachmentHeader()) {
        // Attachment already removed, probably by this program
        if (m_eOutputLevel >= OL_Normal)
            cout << "Attachment was already removed: " << m_pPartHeader->GetFilename() << "\n";
        bRemovedEarlier = true;
    } else {
        // Found attachment to be removed now
        if (m_eOutputLevel >= OL_Normal)
            cout << m_pPartHeader->GetFilename() << "\n";
        m_bRemoved = true;
        if (m_eOutputLevel >= OL_Debug && !m_bInBodyToBeRemoved)
            cout << "cPart::RemoveAttachment:Entering attachment body to be removed\n";
        m_bInBodyToBeRemoved = true;
    }

    // Header finished, save it
    m_pPartHeader->Save (m_pOutStream);

    // Text to replace attachment body
    if (m_bInBodyToBeRemoved) {
        m_pOutStream->WriteLine (m_pPartHeader->GetFilename());
    }

    if (m_eOutputLevel >= OL_Debug)
        cout << "cPart::RemoveAttachment-o:bRemovedEarlier=" << bRemovedEarlier << "\n";
    return bRemovedEarlier;
}
