// =============================================================================
// PartHeader.cpp: Implementation of class cPartHeader
// Filter application, (c) lars.staurset@statkart.no 2003
// =============================================================================

#include <iostream>
#include "PartHeader.h"
#include "Exception.h"

// =========================================================================
// Public functions
// =========================================================================

// ---------------------------------------------------------------------------
// Class administration (construction, destruction, etc.)
// ---------------------------------------------------------------------------
// Constructor
cPartHeader::cPartHeader (const bool cbReport, const OutputLevel ceOutputLevel)
: cHeader (ceOutputLevel)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::cPartHeader\n";
    m_bReport = cbReport;
}

cPartHeader::~cPartHeader(void)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::~cPartHeader\n";
}

// ---------------------------------------------------------------------------
// Data processing
// ---------------------------------------------------------------------------
// Analyze a header when all lines have been read
bool cPartHeader::Analyze ()
{
    m_hpContentType1.nLength = 0;
    m_hpContentType2.nLength = 0;
    m_hpContentDisposition.nLength = 0;
    m_hpContentTransferEncoding.nLength = 0;
    m_sFilename.erase();
    bool bKeepIt = false;

    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::Analyze-i\n";

    // Iterator loop
    for (list<string>::iterator iPos = m_lsLines.begin();
                                iPos != m_lsLines.end(); ++iPos) {
        if (AnalyzeLine (iPos))
            bKeepIt = true;
    }

    // All lines have been analyzed -- now check that Content-Type is present
    if (!m_hpContentType1.nLength)
        throw cException ("Missing Content-Type, probably incomplete part header");

    // Extract file name if possible
    // Even a report part that has been processed before can contain a name here
    if (m_hpContentType2.nLength) {
        m_sFilename = m_hpContentType2.iPos->substr (m_hpContentType2.nPos,
                                                     m_hpContentType2.nLength);
    }

    // Special processing of attachment parts
    if (!m_bReport) {
        // Extract file name from Content-Disposition
        if (m_hpContentDisposition.nLength) {
            string sFilename = m_hpContentDisposition.iPos->substr (m_hpContentDisposition.nPos,
                                                                    m_hpContentDisposition.nLength);
            // Handle ambiguous names (keeping that of Content-Type)
            if (m_hpContentType2.nLength) {
                if (m_sFilename != sFilename) {
                    if (m_eOutputLevel >= OL_Normal) {
                        cout << "Attachment has ambiguous file names:\n";
                        cout << "\'" << sFilename << "\' in Content-Disposition\n";
                        cout << "\'" << m_sFilename << "\' in Content-Type (used here)\n";
                    }
                }
            } else {
                // No name in Content-Type, keep that of Content-Disposition
                m_sFilename = sFilename;
            }
            bKeepIt = false;
        } else if (!m_hpContentType2.nLength) {
            // No name anywhere
            if (m_eOutputLevel >= OL_Debug && !bKeepIt)
                cout << "cPartHeader::Analyze: Keeping part because we don't know what it is\n";
            bKeepIt = true;
        }
    }

    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::Analyze-o: bKeepIt=" << bKeepIt << "\n";
    return bKeepIt;
}

// Prepare attachment header whose body is to be removed
bool cPartHeader::PrepareAttachmentHeader()
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::PrepareAttachmentHeader-i\n";

    // If this program has been run before, the part may already be removed
    bool bRemovedEarlier = false;

    // Pseudo loop (run once)
    do {
        if (m_sFilename.empty())
            break;              // Nothing to do

        // Does file name start with "removed" prefix?
        const string csRemovedPrefix = "Removed attachment: ";
        size_t n = 0;
        if (StringStartsWith (m_sFilename, csRemovedPrefix, n)) {
            bRemovedEarlier = true;
            break;              // Nothing to do
        }

        // Insert prefix to indicate removal
        m_sFilename.insert(0, csRemovedPrefix);

        // Handle Content-Type part 2 before 1 because 1 could invalidate line position
        if (m_hpContentType2.nLength) {
            // Wrap file name in quotes if not done
            if (m_hpContentType2.iPos->at(m_hpContentType2.nPos-1) == '\"')
                m_hpContentType2.iPos->replace (m_hpContentType2.nPos,
                                                m_hpContentType2.nLength,
                                                m_sFilename);
            else
                m_hpContentType2.iPos->replace (m_hpContentType2.nPos,
                                                m_hpContentType2.nLength+2,
                                                "\"" + m_sFilename + "\"");
        }

        // Content-Type part 1
        if (m_hpContentType1.nLength) {
            m_hpContentType1.iPos->replace (m_hpContentType1.nPos,
                                            m_hpContentType1.nLength,
                                            "text/plain;");
        }

        // Content-Disposition
        if (m_hpContentDisposition.nLength) {
            // Wrap file name in quotes if not done
            if (m_hpContentDisposition.iPos->at(m_hpContentDisposition.nPos-1) == '\"')
                m_hpContentDisposition.iPos->replace (m_hpContentDisposition.nPos,
                                                      m_hpContentDisposition.nLength,
                                                      m_sFilename);
            else
                m_hpContentDisposition.iPos->replace (m_hpContentDisposition.nPos,
                                                      m_hpContentDisposition.nLength+2,
                                                      "\"" + m_sFilename + "\"");
        }

        // Content-Transfer-Encoding
        if (m_hpContentTransferEncoding.nLength) {
            m_hpContentTransferEncoding.iPos->replace (m_hpContentTransferEncoding.nPos,
                                                       m_hpContentTransferEncoding.nLength,
                                                       "7bit"); // Is this the optimal value?
        }

    } while (false); // End of pseudo loop

    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::PrepareAttachmentHeader-o: bRemovedEarlier=" << bRemovedEarlier << "\n";
    return bRemovedEarlier;
}

// Prepare report part header whose body is to be removed
bool cPartHeader::PrepareReportPartHeader()
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::PrepareReportPartHeader-i: m_sContentType=\'" << m_sContentType << "\'\n";

    // If this program has been run before, the part may already be removed
    bool bRemovedEarlier = false;

    // Pseudo loop (run once)
    do {
        if (m_sContentType.empty())
            break;              // Nothing to do

        // Does file name start with "removed" prefix?
        const string csRemovedPrefix = "Removed part: ";
        if (m_hpContentType2.nLength) {
            size_t n = 0;
            if (StringStartsWith (m_hpContentType2.iPos->substr (m_hpContentType2.nPos), csRemovedPrefix, n)) {
                bRemovedEarlier = true;
                break;              // Nothing to do
            }
        }

        // Content-Type
        // Before: Content-Type: message/rfc822
        // After:  Content-Type: text/plain; name="Removed message/rfc822"
        if (m_hpContentType1.nLength) {
            m_sContentType.insert (0, csRemovedPrefix);
            m_hpContentType1.iPos->insert (m_hpContentType1.nPos, "text/plain; name=\"" + csRemovedPrefix);
            m_hpContentType1.iPos->append ("\"");
        }

    } while (false); // End of pseudo loop

    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::PrepareReportPartHeader-o: bRemovedEarlier=" << bRemovedEarlier << "\n";
    return bRemovedEarlier;
}

// =========================================================================
// Private functions
// =========================================================================

// Analyze one line of header
bool cPartHeader::AnalyzeLine (list<string>::iterator iPos)
{
    if (iPos->length() == 0)
        return false;           // Empty line, nothing to do
    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::AnalyzeLine-i:\'" << *iPos << "\'\n";
    bool bKeepIt = false;

    // Convert line to lower case for comparisons
    string sLowerCaseLine = *iPos;
    for (unsigned int un = 0; un < sLowerCaseLine.length(); ++un) {
        char ch = sLowerCaseLine[un];
        sLowerCaseLine[un] = tolower (ch);
    }

    // Content-Type may contain content type and file name
    size_t nPos;
    if (StringStartsWith (sLowerCaseLine, "content-type:", nPos)) {
        if (m_eOutputLevel >= OL_Debug)
            cout << "cPartHeader::AnalyzeLine: Found content-type\n";
        m_hpContentType1.nLength = Find (sLowerCaseLine, nPos,
                                            m_hpContentType1.nPos);
        m_hpContentType1.iPos = iPos;

        // Extract Content-Type
        m_sContentType = m_hpContentType1.iPos->substr (m_hpContentType1.nPos,
                                                        m_hpContentType1.nLength);
        if (m_eOutputLevel >= OL_Debug)
            cout << "cPartHeader::AnalyzeLine: Found \'" << m_sContentType << "\'\n";
        
        // Find file name (also in Report if already processed by this program)
        m_hpContentType2.nPos = m_hpContentType1.nPos;      // Starting point for Find
        m_hpContentType2.nLength = Find (sLowerCaseLine, "name", nPos,
                                         m_hpContentType2.nPos);
        m_hpContentType2.iPos = iPos;

    }

    if (m_bReport) {

        // Message-type is Report, keep part if special type
        // (should there be more of these?)
        string sContentType = sLowerCaseLine.substr (m_hpContentType1.nPos,
                                                     m_hpContentType1.nLength);
        if (sContentType == "message/delivery-status" ||
            sContentType == "text/rfc822-headers") {
            if (m_eOutputLevel >= OL_Debug)
                cout << "cPartHeader::AnalyzeLine: Keeping part because of its Report type\n";
            bKeepIt = true;
        }

    } else {

        // Non-report: Content-Disposition may also contain file name
        if (StringStartsWith (sLowerCaseLine, "content-disposition:", nPos)) {
            m_hpContentDisposition.nLength = Find (sLowerCaseLine, "filename", nPos, m_hpContentDisposition.nPos);
            m_hpContentDisposition.iPos = iPos;
        }

        // Content-Transfer-Encoding contains encoding
        if (StringStartsWith (sLowerCaseLine, "content-transfer-encoding:", nPos)) {
            m_hpContentTransferEncoding.nLength = Find (sLowerCaseLine, nPos, m_hpContentTransferEncoding.nPos);
            m_hpContentTransferEncoding.iPos = iPos;
        }
        /*bKeepIt = true;*/
    }

    if (m_eOutputLevel >= OL_Debug)
        cout << "cPartHeader::AnalyzeLine-o: bKeepIt=" << bKeepIt << "\n";
    return bKeepIt;
}